<div class="modal fade" id="swh-web-modal-message" tabindex="-1" role="dialog" aria-labelledby="swh-web-modal-message-label" aria-hidden="true">
  <div class="modal-dialog">
    <div class="modal-content">
      <div class="modal-header">
        <h6 class="modal-title" id="swh-web-modal-message-label"></h6>
        <button type="button" class="close" data-dismiss="modal" aria-label="Close">
          <span aria-hidden="true">&times;</span>
        </button>
      </div>
      <div class="modal-body">
        <p></p>
      </div>
      <div class="modal-footer">
        <button type="button" class="btn btn-default btn-sm" data-dismiss="modal">Ok</button>
      </div>
    </div>
  </div>
</div>

<div class="modal fade" id="swh-web-modal-confirm" tabindex="-1" role="dialog" aria-labelledby="swh-web-modal-confirm-label" aria-hidden="true">
    <div class="modal-dialog">
      <div class="modal-content">
        <div class="modal-header">
          <h6 class="modal-title" id="swh-web-modal-confirm-label"></h6>
          <button type="button" class="close" data-dismiss="modal" aria-label="Close">
            <span aria-hidden="true">&times;</span>
          </button>
        </div>
        <div class="modal-body">
          <p></p>
        </div>
        <div class="modal-footer">
          <button type="button" class="btn btn-default btn-sm" data-dismiss="modal">Cancel</button>
          <button type="button" id="swh-web-modal-confirm-ok-btn" class="btn btn-default btn-sm" data-dismiss="modal">Ok</button>
        </div>
      </div>
    </div>
  </div>
