/**
 * Copyright (C) 2018  The Software Heritage developers
 * See the AUTHORS file at the top-level directory of this distribution
 * License: GNU Affero General Public License version 3, or any later version
 * See top-level LICENSE file for more information
 */

.swh-content {
    background-image: none;
    background-color: white;
    padding: 0;
}

.swh-content pre,
.swh-content pre code {
    margin: 0;
    padding: 0;
}
