/**
 * Copyright (C) 2018  The Software Heritage developers
 * See the AUTHORS file at the top-level directory of this distribution
 * License: GNU Affero General Public License version 3, or any later version
 * See top-level LICENSE file for more information
 */

.swh-browse-nav li a {
    border-radius: 4px;
}

.scrollable-menu {
    max-height: 180px;
    overflow-x: hidden;
}

.swh-corner-ribbon {
    width: 200px;
    background: #fecd1b;
    color: #e20026;
    position: absolute;
    text-align: center;
    line-height: 50px;
    letter-spacing: 1px;
    box-shadow: 0 0 3px rgba(0, 0, 0, 0.3);
    top: 55px;
    right: -50px;
    left: auto;
    transform: rotate(45deg);
    z-index: 2000;
}

.swh-loading {
    display: none;
    text-align: center;
    margin-top: 10px;
}

.swh-loading.show {
    display: block;
}

.swh-metadata-table-row {
    border-top: 1px solid #ddd !important;
}

.swh-metadata-table-key {
    min-width: 200px;
    max-width: 200px;
    width: 200px;
}

.swh-metadata-table-value pre {
    white-space: pre-wrap;
}

.swh-table-cell-text-overflow {
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
}

.swh-directory-table {
    margin-bottom: 0;
}

.swh-directory-table td {
    border-top: 1px solid #ddd !important;
}

.swh-title-color {
    color: #e20026;
}

.swh-log-entry-message {
    min-width: 440px;
    max-width: 440px;
    width: 440px;
}

.swh-popover {
    max-height: 50vh;
    max-width: 80vw;
    overflow-y: auto;
    overflow-x: auto;
    padding: 0;
    padding-right: 1.4em;
}

.swh-search-pagination {
    margin-top: 5px;
}

.ui-slideouttab-panel {
    z-index: 30000;
}

#swh-identifiers {
    width: 70vw;
    top: 0;
    border: 1px solid #e20026;
}

#swh-identifiers .handle {
    background-color: #e20026;
    border: 1px solid #e20026;
    color: white;
    padding-top: 0.1em;
    padding-bottom: 0.1em;
}

#swh-identifiers-content {
    height: 100%;
    overflow: auto;
}

.swh-empty-snapshot {
    white-space: pre-line;
}

td.swh-branch-name {
    max-width: 300px;
}

td.swh-branch-message {
    min-width: 500px;
    max-width: 500px;
}

td.swh-branch-date {
    min-width: 250px;
}

td.swh-branch-name a,
td.swh-branch-message a,
td.swh-branch-date a {
    text-decoration: none;
}

td.swh-release-name a,
td.swh-release-message a,
td.swh-release-date a {
    text-decoration: none;
}

tr.swh-directory-entry a,
tr.swh-search-result-entry a {
    text-decoration: none;
}
