/**
 * Copyright (C) 2018  The Software Heritage developers
 * See the AUTHORS file at the top-level directory of this distribution
 * License: GNU Affero General Public License version 3, or any later version
 * See top-level LICENSE file for more information
 */

.swh-browse-bread-crumbs {
    font-size: inherit;
    vertical-align: text-top;
    margin-bottom: 1px;
}

.swh-browse-bread-crumbs li:nth-child(n+2)::before {
    content: "";
    display: inline-block;
    margin: 0 2px;
}
