.swh-org dl dd{margin-left:9px}.swh-org img,.swh-org table{margin-bottom:1rem}.swh-org table,.swh-org table td,.swh-org table th{border:1px solid #dfe2e5;padding:6px 13px}.swh-org table tr:nth-child(2n){background-color:#f2f2f2}.swh-org span.task-status{color:#fff;margin-right:10px;padding:0 .2em}.swh-org span.task-status.todo{background-color:#2795b6;color:#fff}.swh-org span.task-status.done{background-color:#518f1f;color:#fff}.swh-org .section-number{font-weight:700;padding-right:5px}.swh-org .org-subscript-child{font-size:85%}